/*
    keyhandD.prg

    A simple memoeditor.

    Copyright 1994-1995 by Extrasensory Software.  Permission granted
    to use and distribute these functions freely in source, object, or
    executable form as long as this copyright notice is not removed 
    from the source.
*/

// A simple memoeditor.
//
//   Implements Marks using the shifted arrow keys
//
//   Implements Cut, Copy and Paste using Ctrl+X, Ctrl+C and Ctrl+V
//
//   Call with:
//
//      editor( Top, Left, Bottom, Right, Text )
//
//   Returns:
//  
//      The edited text if saved, otherwise the original text. 
//


#include "inkey.ch"
#include "MrEdit.ch"


#define MEMO_LEN  32000
#define MAX_LINES  3200

function editor( nTop, nLeft, nBottom, nRight, cText )
	local nKey
	local memo
	local lShift
	local lCtrl
	local clip := ""
	local cancel := set(_SET_CANCEL, .f.)
	local clearBlock := .t.


	memo := meCreate( nTop, nLeft, nBottom, nRight, ;
			cText, MEMO_LEN, MAX_LINES, .f. )

	meSetLineWidth(memo, 100)

	meGoTo(memo, 12,1)
	meBlockOn(memo, BLOCK_EXCLUSIVE)
	meGoTo(memo, 17,1)
	@ nTop-1,nLeft-1 to nBottom+1, nRight+1
	@ nTop-1,nLeft say " Read the code to see what you can do! " color "I"
	meDispEnd(memo)

	do while .t. 	

		setcursor(iif(meSetInsert(memo),2,1))

		nKey   := inkey(0)

		setcursor(0)

		if clearBlock
			meBlockOff(memo)
			clearBlock := .f.
		endif

		// Clipper won't give us the state of the Shift and Ctrl keys
		// so we have to improvise and grab them here. Shift is used to
		// implement Shift+Arrow marking. Ctrl is used to make sure that
		// keys like Ctrl+C and PageDown can be told apart.
		lShift := meIsShift()
		lCtrl  := meIsCtrl()

		do case	
			case nKey >= 32 .and. nKey <= 255 .or. nKey == 13
				meapplykey( memo, nKey )

			case nKey == K_INS .and. !lCtrl
				meSetInsert(memo, !meSetInsert(memo))

			case nKey == K_UP .and. !lCtrl
				// If the shift key is down and there is not already a block
				// marked start marking a block
				if lShift .and. !meIsBlock( memo )
					meBlockOn( memo )
				endif
				meUp( memo )

				// we don't want this to fire on Ctrl+X, so don't   	
				// do this if the Ctrl key was down!
			case nKey == K_DOWN .and. !lCtrl
				if lShift .and. !meIsBlock( memo )
					meBlockOn( memo, BLOCK_EXCLUSIVE )
				endif
				meDown( memo )

			case nKey == K_LEFT .and. !lCtrl
				if lShift .and. !meIsBlock( memo )
					meBlockOn( memo, BLOCK_EXCLUSIVE )
				endif
				meLeft( memo )

			case nKey == K_RIGHT .and. !lCtrl
				if lShift .and. !meIsBlock( memo )
					meBlockOn( memo, BLOCK_EXCLUSIVE )
				endif
				meRight( memo )

			case nKey == K_PGUP .and. !lCtrl
				mePageUp( memo )

				// we don't want this to fire on Ctrl+C, so don't   	
				// do this if the Ctrl key was down!
			case nKey == K_PGDN .and. !lCtrl
				mePageDown( memo )

			case nKey == K_HOME .and. !lCtrl
				meHome( memo )

			case nKey == K_END .and. !lCtrl
				meEnd( memo )

			case nKey == K_CTRL_RIGHT .and. lCtrl
				meWordRight( memo )

			case nKey == K_CTRL_LEFT .and. lCtrl
				meWordLeft( memo )

			case nKey == K_CTRL_HOME .and. lCtrl
				meGoTop( memo )

			case nKey == K_CTRL_END .and. lCtrl
				meGoBottom( memo )

			case nKey == K_BS
				meBackSpace( memo )

			case nKey == K_DEL .and. !lCtrl
				meDelete( memo )

			case nKey == K_CTRL_Y .and. lCtrl
				meDelLine(memo)

			case nKey == K_CTRL_T .and. lCtrl
				meDelWordRight(memo)

			case nKey == K_CTRL_U .and. lCtrl
				if meIsBlock( memo )
					meBlockOff( memo )
				endif



				// in the following three cases check to see the Ctrl key
				// was down so we don't get false hits.
			case nKey == K_CTRL_X .and. lCtrl
				// Cut: Capture the text to "clip"
				//      Then delete the marked text
				if meIsBlock( memo )
					clip := megetMarkedBlock( memo )
					medelete( memo )
				endif

			case nKey == K_CTRL_C .and. lCtrl
				// Copy: Capture the text to "clip"
				//       Then turn off the block
				if meIsBlock( memo )
					clip := megetMarkedBlock( memo )
					meBlockOff( memo )
				endif

			case nKey == K_CTRL_V .and. lCtrl
				// Paste: Insert the current scrap
				if !empty( clip )
					meInsertText( memo, clip )
				endif

			case nKey == K_ESC
				exit

			case nKey == K_F10
				cText := megettext( memo )
				exit

		endcase
	enddo
	medestroy( memo )
	set(_SET_CANCEL, cancel)
	return cText
